<?php

namespace Drupal\datadog_metrics\EventSubscriber;

use Drupal\Core\Database\Connection;
use Drupal\Core\State\StateInterface;
use Drupal\datadog_metrics\Utils\MonitoringUtils;
use Drupal\migrate\Event\MigrateEvents;
use Drupal\migrate\Event\MigrateImportEvent;
use Drupal\migrate\Event\MigratePostRowSaveEvent;
use Drupal\migrate\Event\MigrateRowDeleteEvent;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;

class MigrationEventSubscriber implements EventSubscriberInterface {
  use MonitoringUtils;

  private StateInterface $state;
  private Connection $connection;

  public function __construct(StateInterface $state, Connection $connection) {
    $this->connection = $connection;
    $this->state = $state;
  }

  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents(): array {
    $events[MigrateEvents::PRE_IMPORT][] = ['onPreImport'];
    $events[MigrateEvents::POST_ROW_SAVE][] = ['onPostRowSave'];
    $events[MigrateEvents::POST_ROW_DELETE][] = ['onPostRowDelete'];

    return $events;
  }

  public function onPreImport(MigrateImportEvent $event) {
    $migration = $event->getMigration();
    $migrationId = $migration->id();

    foreach (['processed', 'created', 'updated', 'deleted'] as $status) {
      $stateKey = $migrationId . '.' . $status . '_count';
      $this->state->set($stateKey, 0);
    }
  }

  public function onPostRowSave(MigratePostRowSaveEvent $event): void {

    $row = $event->getRow();
    $migration = $event->getMigration();
    $migrationId = $migration->id();

    if (isset($row->getIdMap()["sourceid1"])) {
      $sourceId = $row->getIdMap()["sourceid1"];
    }
    elseif ($row->get('ids')) {
      $idKey = array_key_first($row->get('ids'));
      $sourceId = $row->get($idKey);
    }
    else {
      $sourceId = array_values($row->getSourceIdValues())[0];
    }

    $tableName = "migrate_map_$migrationId";

    $query = $this->connection->select($tableName, 'm')
      ->fields('m', ['sourceid1'])
      ->condition('m.sourceid1', $sourceId);

    $exists = $query->execute()->fetchField();

    if (!$exists) {
      // If there is no entry in the migrate_map table, initialize status to 'created'.
      $status = 'created';
    }
    else {
      // If there is already an entry in the migrate_map table, initialize status to 'updated'.
      $status = 'updated';
    }
    $this->incrementCount($migrationId, $status);
    $this->incrementCount($migrationId, 'processed');
  }

  public function onPostRowDelete(MigrateRowDeleteEvent $event): void {
    $migration = $event->getMigration();
    $migrationId = $migration->id();

    // initialize status to 'deleted'
    $status = 'deleted';
    $this->incrementCount($migrationId, $status);

    // Increment the 'processed' count.
    $this->incrementCount($migrationId, 'processed');
  }

  private function incrementCount($migrationId, $status): void {
    $countKey = $migrationId . '.' . $status . '_count';
    $count = $this->state->get($countKey, 0);
    $count++;
    $this->state->set($countKey, $count);
  }

}
