<?php

namespace Drupal\datadog_metrics;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Config\ImmutableConfig;
use Drupal\Core\Database\Connection;
use Drupal\Core\State\StateInterface;
use Drupal\datadog_metrics\Utils\MetricTypes;
use Drupal\datadog_metrics\Utils\MonitoringUtils;
use Drupal\migrate\Plugin\MigrationPluginManagerInterface;
use GuzzleHttp\ClientInterface;

class MigrationsMonitoring {

  use MonitoringUtils;

  public ClientInterface $httpClient;
  public MigrationPluginManagerInterface $migrationPluginManager;
  public StateInterface $state;
  public ImmutableConfig $datadogConfig;
  private Connection $connection;

  public function __construct(
    ClientInterface $httpClient,
    ConfigFactoryInterface $configFactory,
    MigrationPluginManagerInterface $migrationPluginManager,
    StateInterface $state,
    Connection $connection
  ) {
    $this->httpClient = $httpClient;
    $this->datadogConfig = $configFactory->get('datadog.settings');
    $this->migrationPluginManager = $migrationPluginManager;
    $this->state = $state;
    $this->connection = $connection;
  }

  /**
   * Migrations Monitoring
   *
   * @command migrations-monitoring
   * @aliases mm
   * @usage migrations-monitoring
   */
  public function migrationsMonitoring() {
    $migrations = $this->migrationPluginManager->getDefinitions();
    $monitoringInformation = $this->getMonitoringInformation($migrations);
    $this->submitToDatadog($monitoringInformation, $this->datadogConfig, $this->httpClient);
    $this->resetMigrationsMonitoringStates($migrations);
  }

  public function getMonitoringInformation(array $migrations): array {
    $monitoringInformation = [];

    foreach ($migrations as $migration) {
      $migrationId = $migration['id'];
      $migrationInstance = $this->migrationPluginManager->createInstance($migrationId);
      $migrationStatus = $migrationInstance->getStatus();
      $migrationGroup = $migrationInstance->getPluginDefinition()["migration_group"];
      $tableName = "migrate_map_$migrationId";
      $destinationCount = $this->connection
        ->select($tableName)
        ->countQuery()
        ->execute()
        ->fetchField();

      $metricTags = [
        "host:" . $this->datadogConfig->get('env'),
        "migration:" . $migrationId,
        "migration_group:" . $migrationGroup,
      ];

      $monitoringInformation[] = $this->getMonitoringInformationEntry(
        'migration.status',
        MetricTypes::GAUGE,
        $migrationStatus,
        $metricTags
      );

      $monitoringInformation[] = $this->getMonitoringInformationEntry(
        'migration.items',
        MetricTypes::COUNT,
        (int) $destinationCount,
        $metricTags
      );

      foreach (['processed', 'created', 'updated', 'deleted'] as $status) {
        $monitoringInformation[] = $this->getMonitoringInformationEntry(
          "migration.$status",
          MetricTypes::COUNT,
          $this->state->get($migrationId . '.' . $status . '_count', 0),
          $metricTags
        );
      }

    }

    return $monitoringInformation;
  }

  public function resetMigrationsMonitoringStates(array $migrations): void {
    foreach ($migrations as $migration) {
      $migrationId = $migration['id'];

      foreach (['processed', 'created', 'updated', 'deleted'] as $status) {
        $stateKey = $migrationId . '.' . $status . '_count';
        $this->state->set($stateKey, 0);
      }

    }

  }

}
