<?php

namespace datadog_metrics\tests\Unit;

use Drupal\Core\Config\ConfigFactory;
use Drupal\Core\Config\ImmutableConfig;
use Drupal\Core\Database\Connection;
use Drupal\Core\Database\Query\Select;
use Drupal\Core\Database\StatementWrapper;
use Drupal\Core\State\State;
use Drupal\datadog_metrics\MigrationsMonitoring;
use Drupal\datadog_metrics\Utils\MetricTypes;
use Drupal\migrate\Plugin\Migration;
use Drupal\migrate\Plugin\MigrationPluginManager;
use Drupal\Tests\UnitTestCase;
use GuzzleHttp\Client;

class MigrationsMonitoringTest extends UnitTestCase {

  public $httpClient;
  public $configFactory;
  public $migrationPluginManager;
  public $state;
  public MigrationsMonitoring $migrationsMonitoring;
  public array $migrations;

  protected function setUp(): void {
    parent::setUp();

    $this->migrations = [
      'organization_import' => [
        'id' => 'organization_import',
      ],
    ];

    $this->httpClient = $this->createMock(Client::class);
    $this->configFactory = $this->createMock(ConfigFactory::class);
    $config = $this->getMockBuilder(ImmutableConfig::class)
      ->disableOriginalConstructor()
      ->getMock();

    $config->method('get')
      ->with('env')
      ->willReturn('local');

    $this->configFactory
      ->method('get')
      ->with('datadog.settings')
      ->willReturn($config);

    $this->migrationPluginManager = $this->createMock(MigrationPluginManager::class);
    $this->state = $this->createMock(State::class);
    $this->connection = $this->createMock(Connection::class);
    $this->migrationsMonitoring = new MigrationsMonitoring($this->httpClient, $this->configFactory, $this->migrationPluginManager, $this->state, $this->connection);

  }

  public function testGetMonitoringInformation() {
    $migrationInstance = $this->createMock(Migration::class);
    $migrationInstance->method('getStatus')->willReturn(0);
    $migrationInstance->method('getPluginDefinition')->willReturn([
      'migration_group' => 'bs_state_calendar',
    ]);

    $this->migrationPluginManager
      ->method('createInstance')
      ->willReturn($migrationInstance);

    $this->state
      ->method('get')
      ->will(
        $this->returnValueMap(
          [
            ['organization_import.processed_count', 0, 52],
            ['organization_import.created_count', 0, 8],
            ['organization_import.updated_count', 0, 41],
            ['organization_import.deleted_count', 0, 3],
          ]
        )
      );

    $statementWrapper = $this->createMock(StatementWrapper::class);
    $statementWrapper
      ->expects($this->once())
      ->method('fetchField')
      ->willReturn('2113');

    $select = $this->createMock(Select::class);
    $select
      ->expects($this->once())
      ->method('countQuery')
      ->willReturn($select);

    $select
      ->expects($this->once())
      ->method('execute')
      ->willReturn($statementWrapper);

    $this->connection
      ->expects($this->once())
      ->method('select')
      ->willReturn($select);

    $actualMonitoringInformation = $this->migrationsMonitoring->getMonitoringInformation($this->migrations);

    $metricTags = [
      "host:" . 'local',
      "migration:" . 'organization_import',
      "migration_group:" . 'bs_state_calendar',
    ];

    $expectedMonitoringInformation = [
      [
        'metric_name' => 'migration.status',
        'metric_type' => MetricTypes::GAUGE,
        'metric_value' => 0,
        'metric_tags' => $metricTags,
      ],
      [
        'metric_name' => 'migration.items',
        'metric_type' => MetricTypes::COUNT,
        'metric_value' => 2113,
        'metric_tags' => $metricTags,
      ],
      [
        'metric_name' => 'migration.processed',
        'metric_type' => MetricTypes::COUNT,
        'metric_value' => 52,
        'metric_tags' => $metricTags,
      ],
      [
        'metric_name' => 'migration.created',
        'metric_type' => MetricTypes::COUNT,
        'metric_value' => 8,
        'metric_tags' => $metricTags,
      ],
      [
        'metric_name' => 'migration.updated',
        'metric_type' => MetricTypes::COUNT,
        'metric_value' => 41,
        'metric_tags' => $metricTags,
      ],
      [
        'metric_name' => 'migration.deleted',
        'metric_type' => MetricTypes::COUNT,
        'metric_value' => 3,
        'metric_tags' => $metricTags,
      ],

    ];

    $this->assertEquals($expectedMonitoringInformation, $actualMonitoringInformation);

  }

  public function testResetMigrationsMonitoringStates() {
    $this->state
      ->expects($this->exactly(4))
      ->method('set')
      ->with($this->logicalOr(
        $this->equalTo('organization_import.processed_count'),
        $this->equalTo('organization_import.created_count'),
        $this->equalTo('organization_import.updated_count'),
        $this->equalTo('organization_import.deleted_count'),
      ),
        $this->equalTo(0)
      );

    $this->migrationsMonitoring->resetMigrationsMonitoringStates($this->migrations);

  }

}
